`include "constants.v"

module fifo(
  clk, tftclk,
  r, g, b,
  layers_ready,
  rearm_layers,
  underflow,
  raster_x, raster_y,
  lcd_de, lcd_vsync, lcd_hsync, lcd_r, lcd_g, lcd_b
);

input clk;
input tftclk;
input [7:0] r;
input [7:0] g;
input [7:0] b;
input layers_ready;
output rearm_layers;
output underflow;
output reg [8:0] raster_x = 9'b0;
output reg [8:0] raster_y = 9'b0;
output [7:0] lcd_r;
output [7:0] lcd_g;
output [7:0] lcd_b;
output lcd_hsync;
output lcd_vsync;
output lcd_de;

wire rgb_fifo_full;
wire rgb_fifo_empty;
wire rgb_fifo_has_one_line;

assign rearm_layers = clk & layers_ready & ~rgb_fifo_full;
assign underflow = rgb_fifo_empty;

wire [7:0] fifo_out_r;
wire [7:0] fifo_out_g;
wire [7:0] fifo_out_b;

reg fifo_rst = 1'b1;

rgb_fifo rbg_fifo_instance (
  .rst(fifo_rst),
  .wr_clk(~clk),
  .rd_clk(tftclk),
  .din({r, g, b}),
  .wr_en(layers_ready),
  .rd_en(lcd_de),
  .dout({fifo_out_r, fifo_out_g, fifo_out_b}),
  .full(rgb_fifo_full),
  .empty(rgb_fifo_empty),
  .prog_full(rgb_fifo_has_one_line)
);

always @(posedge clk) begin
  fifo_rst <= 0;
  if (!rgb_fifo_full) begin
    if (layers_ready) begin
      if (raster_x == `H_DISPLAY) begin
        raster_x <= 0;
        if (raster_y == ` V_DISPLAY) begin
          raster_y <= 0;
        end else raster_y <= raster_y + 1;
      end else raster_x <= raster_x + 1;
    end
  end
end

reg [1:0] vstate = `STATE_PULSE_WIDTH;
reg [1:0] hstate = `STATE_PULSE_WIDTH;
reg [8:0] vcounter = `V_PULSE_WIDTH;
reg [8:0] hcounter = `H_PULSE_WIDTH;

assign lcd_de = vstate == `STATE_DISPLAY && hstate == `STATE_DISPLAY;
assign lcd_hsync = hstate == `STATE_PULSE_WIDTH;
assign lcd_vsync = vstate == `STATE_PULSE_WIDTH;

assign lcd_r = fifo_out_r;
assign lcd_g = fifo_out_g;
assign lcd_b = fifo_out_b;

always @(posedge tftclk) begin
  if (!lcd_de || !rgb_fifo_empty) begin
    if (hcounter == 0) begin
      hstate = hstate + 1;
      case(hstate)
        `STATE_PULSE_WIDTH: begin
          hcounter <= `H_PULSE_WIDTH;
          if (vcounter == 0) begin
            vstate = vstate + 1;
            case(vstate)
              `STATE_PULSE_WIDTH: vcounter <= `V_PULSE_WIDTH;
              `STATE_BACK_PORCH:  vcounter <= `V_BACK_PORCH;
              `STATE_DISPLAY:     vcounter <= `V_DISPLAY;
              `STATE_FRONT_PORCH: vcounter <= `V_FRONT_PORCH;
            endcase
          end else vcounter <= vcounter - 1;
        end
        `STATE_BACK_PORCH:  hcounter <= `H_BACK_PORCH;
        `STATE_DISPLAY:     hcounter <= `H_DISPLAY;
        `STATE_FRONT_PORCH: hcounter <= `H_FRONT_PORCH;
      endcase
    end else hcounter <= hcounter - 1;
  end
end

endmodule
