`include "constants.v"

module text_layer(
  clk,
  text_wr_col,
  text_wr_line,
  text_wr_size,
  text_wr_char,
  text_wr_en,
  font_wr_char,
  font_wr_x,
  font_wr_y,
  font_wr_value,
  font_wr_en,
  raster_x, raster_y,
  rearm_layer,
  r, g, b, a,
  layer_ready
);

input clk;
input [8:0] raster_x;
input [8:0] raster_y;
input rearm_layer;
input [6:0] text_wr_col;
input [6:0] text_wr_line;
input text_wr_size;
input [7:0] text_wr_char;
input text_wr_en;
input [7:0] font_wr_char;
input [2:0] font_wr_x;
input [2:0] font_wr_y;
input font_wr_value;
input font_wr_en;
output [7:0] r;
output [7:0] g;
output [7:0] b;
output [7:0] a;
output reg layer_ready;

wire [12:0] text_fb_rd_addr;
wire text_fb_rd_size;
wire [7:0] text_fb_rd_char;

wire [10:0] text_fb_wr_addr;
assign text_fb_wr_addr = text_wr_col + text_wr_line * `H_DISPLAY / 8;

text_fb text_fb_instance (
  .clka(clk),
  .wea(text_wr_en),
  .addra(text_fb_wr_addr),
  .dina({text_wr_size, text_wr_char}),
  .clkb(clk),
  .addrb(text_fb_rd_addr),
  .doutb({text_fb_rd_size, text_fb_rd_char})
);

wire [13:0] vram_font_wr_addr;
assign vram_font_wr_addr = font_wr_char * 64 + font_wr_y * 8 + font_wr_x;

wire [13:0] vram_font_rd_addr;
wire vram_font_rd_bit;

vram_font vram_font_instance (
  .clka(clk),
  .wea(font_wr_en),
  .addra(vram_font_wr_addr),
  .dina(font_wr_value),
  .clkb(clk),
  .addrb(vram_font_rd_addr),
  .doutb(vram_font_rd_bit)
);

always @(posedge clk) if (!layer_ready) layer_ready <= 1;
else if (rearm_layer) layer_ready <= 0;

assign r = vram_font_rd_bit ? 8'hff : 8'h00;
assign g = vram_font_rd_bit ? 8'hff : 8'h00;
assign b = vram_font_rd_bit ? 8'hff : 8'h00;
assign a = vram_font_rd_bit ? 8'hff : 8'h00;

assign text_fb_rd_addr = (raster_x >> 3) + (raster_y >> 3) * (`H_DISPLAY >> 3);
assign vram_font_rd_addr = text_fb_rd_char * 64 + (text_fb_rd_size ?
  ((raster_x >> 1) & 7) + ((raster_y >> 1) & 7) * 8 :
  (raster_x & 7) + (raster_y & 7) * 8);

endmodule
