`include "constants.v"

module uGPU(
  clk_pin,
  lcd_r, lcd_g, lcd_b, lcd_dclk, lcd_hsync, lcd_vsync, lcd_de
);

input clk_pin;

output [7:0] lcd_r;
output [7:0] lcd_g;
output [7:0] lcd_b;
output lcd_dclk;
output lcd_hsync;
output lcd_vsync;
output lcd_de;

wire clk;
wire sdclk;
wire tftclk;

//--------
reg [6:0] text_wr_col = 0;
reg [6:0] text_wr_line = 0;
reg text_wr_size = 0;
reg [7:0] text_wr_char = 0;
reg text_wr_en = 1; 
reg [31:0] cnt = 0;
always @(posedge clk) cnt <= cnt + 1;
always @(posedge clk)
case(cnt[31:25])
4'b0001: begin text_wr_col <=  0; text_wr_char <= "H"; end
4'b0010: begin text_wr_col <=  1; text_wr_char <= "e"; end
4'b0011: begin text_wr_col <=  2; text_wr_char <= "l"; end
4'b0100: begin text_wr_col <=  3; text_wr_char <= "l"; end
4'b0101: begin text_wr_col <=  4; text_wr_char <= "o"; end
4'b0110: begin text_wr_col <=  5; text_wr_char <= " "; end
4'b0111: begin text_wr_col <=  6; text_wr_char <= "W"; end
4'b1000: begin text_wr_col <=  7; text_wr_char <= "o"; end
4'b1001: begin text_wr_col <=  8; text_wr_char <= "r"; end
4'b1010: begin text_wr_col <=  9; text_wr_char <= "l"; end
4'b1011: begin text_wr_col <= 10; text_wr_char <= "d"; end
4'b1111: begin text_wr_col <= 11; text_wr_char <= "!"; end
endcase
//--------

wire [7:0] fifo_r;
wire [7:0] fifo_g;
wire [7:0] fifo_b;

wire [8:0] raster_x;
wire [8:0] raster_y;

wire all_layers_ready;
wire text_layer_ready;
wire rearm_layers;

assign all_layers_ready = text_layer_ready;

reg [7:0] background_r = 0;
reg [7:0] background_g = 0;
reg [7:0] background_b = 0;

wire [7:0] text_r;
wire [7:0] text_g;
wire [7:0] text_b;
wire [7:0] text_a;

text_layer text_layer_instance(
  .clk(clk),
  .text_wr_col(text_wr_col),
  .text_wr_line(text_wr_line),
  .text_wr_size(text_wr_size),
  .text_wr_char(text_wr_char),
  .text_wr_en(text_wr_en),
  .font_wr_char(),
  .font_wr_x(),
  .font_wr_y(),
  .font_wr_value(),
  .font_wr_en(0),
  .raster_x(raster_x), .raster_y(raster_y),
  .rearm_layer(rearm_layers),
  .r(text_r), .g(text_g), .b(text_b), .a(text_a),
  .layer_ready(text_layer_ready)
);

assign fifo_r = text_r;  
assign fifo_g = text_g;  
assign fifo_b = text_b;  

fifo fifo_instance(
  .clk(clk),
  .tftclk(tftclk),
  .r(fifo_r),
  .g(fifo_g),
  .b(fifo_b),
  .layers_ready(all_layers_ready),
  .rearm_layers(rearm_layers),
  .raster_x(raster_x),
  .raster_y(raster_y),
  .lcd_r(lcd_r),
  .lcd_g(lcd_g),
  .lcd_b(lcd_b),
  .lcd_vsync(lcd_vsync),
  .lcd_hsync(lcd_hsync),
  .lcd_de(lcd_de)
);

clocks clocks_instance(.clk_pin(clk_pin), .clk(clk), .tft_clk(tftclk));
// tying the clock instance output to the clock IOB
ODDR2 #(.DDR_ALIGNMENT("NONE"), .INIT(1'b0), .SRTYPE("SYNC"))
oddr2_tft_clk(.Q(lcd_dclk), .C0(tftclk), .C1(~tftclk), .CE(1'b1), .D0(1'b1), .D1(1'b0), .R(1'b0), .S(1'b0));

endmodule
